local _G = getfenv(0)

local YurrCombatLog = LibStub("AceAddon-3.0"):NewAddon("YurrCombatLog", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("YurrCombatLog", true)

local debugf = tekDebug and tekDebug:GetFrame("YurrCombatLog")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", ...)) end end

local CombatLogQuickButtonFrameProgressBar
--local CombatLogUpdateFrame = CreateFrame("Frame", "CombatLogUpdateFrame", UIParent)
local defaults = {
	profile = {
		outputFrame = "ChatFrame3",
		unitColors = {
			[1297] = "8080FF", -- self
			[4991] = "804D26", -- pet
			[1394] = "A6FFA6", -- party
			[1396] = "FF7038", -- raid
			[9087] = "404080", -- guardian
			[131072] = "CC54D9", -- focus
			[20334] = "FF75FF", -- enemy players/npcs
			[3864] = "FFFFFF", -- outsider
		},
		spellColors = {
			[1] = "FFFFFF", -- physical
			[2] = "FFFF4D", -- holy
			[4] = "FF262E", -- fire
			[8] = "66FF66", -- nature
			[9] = "B37FB2", -- stormstrike(physical + nature)
			[16] = "4D4DE6", -- frost
			[20] = "A63B96", -- frostfire(frost + fire)
			[24] = "5AA6A6", -- froststorm(frost + nature)
			[32] = "FFB3FF", -- shadow
			[40] = "B3D9B3", -- shadowstorm(shadow + nature)
			[64] = "BFBFBF", -- arcane
			[66] = "E9E986", -- divine(arcane + holy)
			[68] = "DF72F6", -- spellfire(fire + arcane)
			[80] = "8686D3" -- spellfrost(frost + arcane)
		},
		otherColors = {
			heal = "FFFF4D",
			miss = "FF7F7F",
			buff = "33FF33",
			debuff = "FF3333",
			skill = "FFFF66",
			timestampColor = "979797",
			
		},
		blacklist = {},
		watches = {},
		braces = {
			preTimestamp = "[",
			postTimestamp = "]",
			preSource = "[",
			postSource = "]",
			preSpell = "[",
			postSpell = "]",
			preDest = "[",
			postDest = "]",
			preAmount = "[",
			postAmount = "]",
			preAmountDots = "~",
			postAmountDots = "~",
			preSchoolText = "[",
			postSchoolText = "]",
			preCrit = "*",
			postCrit = "*",
		},
		braceColors = {
			Timestamp = "979797",
			Source = "FFFFFF",
			Spell = "FFFFFF",
			Dest = "FFFFFF",
			Amount = "FFFFFF",
			AmountDots = "FFFFFF",
		},
		
		timestamp = true,
		colorTimestamp = true,
		braceTimestamp = true,
		colorTimestampBrace = true,
		timestampFormat = "%X",
		
		braceSrcUnit = true,
		colorSrc = true,
		colorSrcBraces = false,
		
		prefix = true,
		colorPrefix = true,
		["++"] = "++",
		["--"] = "--",
		["~~"] = "~~",
		
		braceSpell = false,
		colorSpell = true,
		colorSpellBraces = false,
		
		suffixString = true,
		resultInSuffix = true,
		
		braceDestUnit = true,
		colorDest = true,
		colorDestBraces = false,
		
		braceAmount = true,
		colorAmount = true,
		colorAmountBraces = false,
		braceAmountDots = true,
		colorAmountDotBraces = false,
		markCrits = true,
		
		overhealing = true,
		overkill = true,
		
		resultString = true,
		
		powerType = true,
		
		schoolText = false,
		colorSchoolText = true,
		braceSchoolText = false,
		colorSchoolTextBraces = false,
		
		format = {
			ABSORB = "(A%d)",
			BLOCK = BLOCK,
			BLOCK_VALUE = "(B%d)",
			CRITICAL = L["Crit"],
			CRUSHING = L["Crush"],
			DEFLECT = DEFLECT,
			DODGE = DODGE,
			EVADE = L["Evade"],
			GLANCING = L["Glancing"],
			IMMUNE = SPELL_FAILED_IMMUNE,
			MISS = MISS,
			PARRY = PARRY,
			REFLECT = REFLECT,
			RESIST = "(R%d)",
			VULNERABILITY = "(V%d)",
		},
		filters = {
			[1] = {
				name = "By me",
				active = true,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = false,
				SPELL_CAST_SUCCESS = false,
				SPELL_CAST_FAILED = false,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = false,
				SPELL_DURABILITY_DAMAGE_ALL = false,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = false,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = false,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = false,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
			},
			[2] = {
				name = "To me",
				active = true,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = false,
				SPELL_CAST_SUCCESS = false,
				SPELL_CAST_FAILED = false,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = false,
				SPELL_DURABILITY_DAMAGE_ALL = false,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = false,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = false,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = false,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
					COMBATLOG_OBJECT_AFFILIATION_RAID = false,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
					COMBATLOG_OBJECT_REACTION_HOSTILE = false,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = false,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = false,
					COMBATLOG_OBJECT_TYPE_PET = false,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
					COMBATLOG_OBJECT_TYPE_OBJECT = false,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = false,
					COMBATLOG_OBJECT_FOCUS = false,
					COMBATLOG_OBJECT_MAINTANK = false,
					COMBATLOG_OBJECT_MAINASSIST = false,
					COMBATLOG_OBJECT_RAIDTARGET1 = false,
					COMBATLOG_OBJECT_RAIDTARGET2 = false,
					COMBATLOG_OBJECT_RAIDTARGET3 = false,
					COMBATLOG_OBJECT_RAIDTARGET4 = false,
					COMBATLOG_OBJECT_RAIDTARGET5 = false,
					COMBATLOG_OBJECT_RAIDTARGET6 = false,
					COMBATLOG_OBJECT_RAIDTARGET7 = false,
					COMBATLOG_OBJECT_RAIDTARGET8 = false,
					COMBATLOG_OBJECT_NONE = false,
				},
			},
			[3] = {
				name = "Everything",
				active = false,
				ENVIRONMENTAL_DAMAGE = true,
				SWING_DAMAGE = true,
				SWING_MISSED = true,
				RANGE_DAMAGE = true,
				RANGE_MISSED = true,
				SPELL_CAST_START = true,
				SPELL_CAST_SUCCESS = true,
				SPELL_CAST_FAILED = true,
				SPELL_MISSED = true,
				SPELL_DAMAGE = true,
				SPELL_HEAL = true,
				SPELL_ENERGIZE = true,
				SPELL_DRAIN = true,
				SPELL_LEECH = true,
				SPELL_SUMMON = true,
				SPELL_RESURRECT = true,
				SPELL_CREATE = true,
				SPELL_INSTAKILL = true,
				SPELL_INTERRUPT = true,
				SPELL_EXTRA_ATTACKS = true,
				SPELL_DURABILITY_DAMAGE = true,
				SPELL_DURABILITY_DAMAGE_ALL = true,
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_APPLIED_DOSE = true,
				SPELL_AURA_REMOVED = true,
				SPELL_AURA_REMOVED_DOSE = true,
				SPELL_AURA_BROKEN = true,
				SPELL_AURA_BROKEN_SPELL = true,
				SPELL_AURA_REFRESH = true,
				SPELL_DISPEL = true,
				SPELL_STOLEN = true,
				ENCHANT_APPLIED = true,
				ENCHANT_REMOVED = true,
				SPELL_PERIODIC_MISSED = true,
				SPELL_PERIODIC_DAMAGE = true,
				SPELL_PERIODIC_HEAL = true,
				SPELL_PERIODIC_ENERGIZE = true,
				SPELL_PERIODIC_DRAIN = true,
				SPELL_PERIODIC_LEECH = true,
				SPELL_DISPEL_FAILED = true,
				DAMAGE_SHIELD = true,
				DAMAGE_SHIELD_MISSED = true,
				DAMAGE_SPLIT = true,
				PARTY_KILL = true,
				UNIT_DIED = true,
				UNIT_DESTROYED = true,
				SPELL_BUILDING_DAMAGE = true,
				SPELL_BUILDING_HEAL = true,
				sourceUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = true,
					COMBATLOG_OBJECT_AFFILIATION_RAID = true,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = true,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = true,
					COMBATLOG_OBJECT_REACTION_HOSTILE = true,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = true,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = true,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = true,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = true,
					COMBATLOG_OBJECT_FOCUS = true,
					COMBATLOG_OBJECT_MAINTANK = true,
					COMBATLOG_OBJECT_MAINASSIST = true,
					COMBATLOG_OBJECT_RAIDTARGET1 = true,
					COMBATLOG_OBJECT_RAIDTARGET2 = true,
					COMBATLOG_OBJECT_RAIDTARGET3 = true,
					COMBATLOG_OBJECT_RAIDTARGET4 = true,
					COMBATLOG_OBJECT_RAIDTARGET5 = true,
					COMBATLOG_OBJECT_RAIDTARGET6 = true,
					COMBATLOG_OBJECT_RAIDTARGET7 = true,
					COMBATLOG_OBJECT_RAIDTARGET8 = true,
					COMBATLOG_OBJECT_NONE = true,
				},
				destUnit = {
					--Controller affiliation
					COMBATLOG_OBJECT_AFFILIATION_MINE = true,
					COMBATLOG_OBJECT_AFFILIATION_PARTY = true,
					COMBATLOG_OBJECT_AFFILIATION_RAID = true,
					COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = true,
					--Reaction
					COMBATLOG_OBJECT_REACTION_FRIENDLY = true,
					COMBATLOG_OBJECT_REACTION_NEUTRAL = true,
					COMBATLOG_OBJECT_REACTION_HOSTILE = true,
					--Controller
					COMBATLOG_OBJECT_CONTROL_PLAYER = true,
					COMBATLOG_OBJECT_CONTROL_NPC = true,
					--Type
					COMBATLOG_OBJECT_TYPE_PLAYER = true,
					COMBATLOG_OBJECT_TYPE_NPC = true,
					COMBATLOG_OBJECT_TYPE_PET = true,
					COMBATLOG_OBJECT_TYPE_GUARDIAN = true,
					COMBATLOG_OBJECT_TYPE_OBJECT = true,
					--Special cases (non-exclusive)
					COMBATLOG_OBJECT_TARGET = true,
					COMBATLOG_OBJECT_FOCUS = true,
					COMBATLOG_OBJECT_MAINTANK = true,
					COMBATLOG_OBJECT_MAINASSIST = true,
					COMBATLOG_OBJECT_RAIDTARGET1 = true,
					COMBATLOG_OBJECT_RAIDTARGET2 = true,
					COMBATLOG_OBJECT_RAIDTARGET3 = true,
					COMBATLOG_OBJECT_RAIDTARGET4 = true,
					COMBATLOG_OBJECT_RAIDTARGET5 = true,
					COMBATLOG_OBJECT_RAIDTARGET6 = true,
					COMBATLOG_OBJECT_RAIDTARGET7 = true,
					COMBATLOG_OBJECT_RAIDTARGET8 = true,
					COMBATLOG_OBJECT_NONE = true,
				},
			},
		},
	}
}

function YurrCombatLog:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("YurrCombatLogDB", defaults, true)
	if self.db.profile.allSource then self:ResetDB("profile") end
	self:RegisterChatCommand("yurrcombatlog", "showoptionsgui")
	self:RegisterChatCommand("ycl", "showoptionsgui")
end

local s = false
function YurrCombatLog:showoptionsgui()
	if not IsAddOnLoaded("YurrCombatLogOptions") then
		local _, reason = LoadAddOn("YurrCombatLogOptions")
		local _, title = GetAddOnInfo("YurrCombatLogOptions")
		if (not title) then title = "YurrCombatLogOptions" end
		if reason then
			self:Print( title .. " (Enable) LoadOnDemand Error - " .. reason )
		end
    end
	if s then
		LibStub("AceConfigDialog-3.0"):Close("YurrCombatLog")
	else
		LibStub("AceConfigDialog-3.0"):Open("YurrCombatLog")
	end
	s = not s
end

local ldb = LibStub("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject("YurrCombatLogOptions", {icon = "Interface\\Icons\\Ability_MeleeDamage", text = "YurrCombatLog"})

function dataobj.OnTooltipShow(self)
	self:AddLine(dataobj.text)
	self:AddLine("|cffffff00"..L.leftClick)
end

dataobj.OnClick = function() YurrCombatLog:showoptionsgui() end

--[[
[SCHOOL_MASK_PHYSICAL]						= STRING_SCHOOL_PHYSICAL,
[SCHOOL_MASK_HOLY]							= STRING_SCHOOL_HOLY,
[SCHOOL_MASK_FIRE]							= STRING_SCHOOL_FIRE,
[SCHOOL_MASK_NATURE]						= STRING_SCHOOL_NATURE,
[SCHOOL_MASK_FROST]							= STRING_SCHOOL_FROST,
[SCHOOL_MASK_SHADOW]						= STRING_SCHOOL_SHADOW,
[SCHOOL_MASK_ARCANE]						= STRING_SCHOOL_ARCANE,
-- Physical and a Magical
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]	= STRING_SCHOOL_FLAMESTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]	= STRING_SCHOOL_FROSTSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_STORMSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTRIKE,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_HOLYSTRIKE,
-- Two Magical Schools
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]		= STRING_SCHOOL_FROSTFIRE,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]		= STRING_SCHOOL_SPELLFIRE,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]		= STRING_SCHOOL_FIRESTORM,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]		= STRING_SCHOOL_SHADOWFLAME,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFIRE,
[SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]	= STRING_SCHOOL_SPELLFROST,
[SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_FROSTSTORM,
[SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWFROST,
[SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYFROST,
[SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]	= STRING_SCHOOL_SPELLSTORM,
[SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SPELLSHADOW,
[SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_DIVINE,
[SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]	= STRING_SCHOOL_SHADOWSTORM,
[SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_HOLYSTORM,
[SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]		= STRING_SCHOOL_SHADOWLIGHT,
-- Three or more schools
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]																						= STRING_SCHOOL_ELEMENTAL,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]											= STRING_SCHOOL_CHROMATIC,
[SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]						= STRING_SCHOOL_MAGIC,
[SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]	= STRING_SCHOOL_CHAOS,]]

local schools = {
	[1] = SPELL_SCHOOL0_NAME, -- physical
	[2] = SPELL_SCHOOL1_NAME, -- holy
	[4] = SPELL_SCHOOL2_NAME, -- fire
	[8] = SPELL_SCHOOL3_NAME, -- nature
	[9] = SPELL_SCHOOL0_NAME..SPELL_SCHOOL3_NAME, -- stormstrike(physical + nature)
	[16] = SPELL_SCHOOL4_NAME, -- frost
	[20] = SPELL_SCHOOL4_NAME..SPELL_SCHOOL2_NAME, -- frostfire(frost + fire)
	[24] = SPELL_SCHOOL4_NAME..SPELL_SCHOOL3_NAME, -- froststorm(frost + nature)
	[32] = SPELL_SCHOOL5_NAME, -- shadow
	[40] = SPELL_SCHOOL3_NAME..SPELL_SCHOOL5_NAME, -- shadowstorm(shadow + nature)
	[64] = SPELL_SCHOOL6_NAME, -- arcane
	[66] = SPELL_SCHOOL6_NAME..SPELL_SCHOOL1_NAME, -- divine(arcane + holy)
	[68] = SPELL_SCHOOL2_NAME..SPELL_SCHOOL6_NAME, -- spellfire(fire + arcane)
	[80] = SPELL_SCHOOL4_NAME..SPELL_SCHOOL6_NAME -- spellfrost(frost + arcane)
}
local powerTypes = {
	[-2] = string.lower(HEALTH),
	[0] = string.lower(MANA),
	[1] = string.lower(RAGE),
	[2] = string.lower(FOCUS),
	[3] = string.lower(ENERGY),
	[4] = string.lower(PET .. HAPPINESS),
	[5] = string.lower(RUNES),
	[6] = string.lower(RUNIC_POWER),
}

function YurrCombatLog:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT")
	
	self:RawHook("Blizzard_CombatLog_Refilter", true)
	self:RawHook("Blizzard_CombatLog_RefilterUpdate", true)
	self:RawHook("CombatLog_AddEvent", true)
	self:RawHook("Blizzard_CombatLog_ApplyFilters", "setFilter", true)
	
	-- ninjad from Prat_ArmoryLink
	self:SecureHook("UnitPopup_ShowMenu")
    UnitPopupButtons['ADD_TO_YCL'] = { 
        text = L["Watch in (Yurr)CombatLog"], 
        dist = 0, 
        func = function(info) YurrCombatLog:UnitPopup_Click(info) end,
    }
    UnitPopupButtons['REMOVE_FROM_YCL'] = { 
        text = L["Remove watch from (Yurr)CombatLog"],
        dist = 0, 
        func = function(info) YurrCombatLog:UnitPopup_Click(info) end,
    }
    local popupList = { 
        'PARTY',        -- party frame.
        'FRIEND',       -- friend frame
        'SELF',         -- ?
        'PLAYER',       -- ?
        'RAID',         -- raid frames
        'RAID_PLAYER',  -- player's raid frame?
        'TEAM'          -- arena team
    }
    for key,val in pairs(popupList) do
        tinsert(UnitPopupMenus[val], #UnitPopupMenus[val]-1,'ADD_TO_YCL')
        tinsert(UnitPopupMenus[val], #UnitPopupMenus[val]-1,'REMOVE_FROM_YCL')
    end
	
	CombatLogQuickButtonFrameProgressBar = _G.CombatLogQuickButtonFrame_CustomProgressBar
	--self:setFilter()
end
-- ninjad from Prat_ArmoryLink
function YurrCombatLog:UnitPopup_Click(info)
	local name = UIDROPDOWNMENU_INIT_MENU.name
	local button = info.value
	local click = {
		["ADD_TO_YCL"] = function()
			self.db.profile.watches[name] = UnitGUID(name)
			if IsAddOnLoaded("YurrCombatLogOptions") then
				local module = self:GetModule("YurrCombatLogOptions")
				local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
				options.args.watches.args[name] = module:createWatch(name, UnitGUID(name))
				AceConfigRegistry:NotifyChange("YurrCombatLog")
			end
			self:setFilter()
		end,
		["REMOVE_FROM_YCL"] = function()
			self.db.profile.watches[name] = nil
			if IsAddOnLoaded("YurrCombatLogOptions") then
				local module = self:GetModule("YurrCombatLogOptions")
				local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
				options.args.watches.args[name] = nil
				AceConfigRegistry:NotifyChange("YurrCombatLog")
			end
			self:setFilter()
		end
	}
	if click[button] then click[button]() end
end
function YurrCombatLog:UnitPopup_ShowMenu(dropdownMenu, which, unit, name, userData, ...)
	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i);

		-- Patch our handler function back in
		if  button.value == "ADD_TO_YCL" then
		    button.func = UnitPopupButtons["ADD_TO_YCL"].func
		elseif  button.value == "REMOVE_FROM_YCL" then
		    button.func = UnitPopupButtons["REMOVE_FROM_YCL"].func
		end
	end
end

function YurrCombatLog:Blizzard_CombatLog_Refilter()
	local count = CombatLogGetNumEntries();
	
	COMBATLOG:SetMaxLines(COMBATLOG_MESSAGE_LIMIT);
	_G[self.db.profile.outputFrame]:SetMaxLines(COMBATLOG_MESSAGE_LIMIT)

	-- count should be between 1 and COMBATLOG_MESSAGE_LIMIT
	count = max(1, min(count, COMBATLOG_MESSAGE_LIMIT));

	CombatLogSetCurrentEntry(0);
	
	-- Clear the combat log
	--COMBATLOG:Clear();
	_G[self.db.profile.outputFrame]:Clear()
	
	-- Moved setting the max value here, since we don't really need to reset the max every frame, do we?
	-- We can't add events while refiltering (:AddFilter short circuits) so this should be safe optimization.
	CombatLogQuickButtonFrameProgressBar:SetMinMaxValues(0, count);	
	CombatLogQuickButtonFrameProgressBar:SetValue(0);
	CombatLogQuickButtonFrameProgressBar:Show();

	-- Enable the distributed frame
	CombatLogUpdateFrame.refiltering = true;
	CombatLogUpdateFrame:SetScript("OnUpdate", Blizzard_CombatLog_RefilterUpdate)	
	
	Blizzard_CombatLog_RefilterUpdate()
end
function YurrCombatLog:Blizzard_CombatLog_RefilterUpdate()
	local valid = CombatLogGetCurrentEntry(); -- CombatLogAdvanceEntry(0);
	
	-- Clear the combat log
	local total = 0;
	while (valid and total < COMBATLOG_LIMIT_PER_FRAME) do 
		-- Log to the window
		--local text, r, g, b, a = CombatLog_OnEvent(Blizzard_CombatLog_CurrentSettings, CombatLogGetCurrentEntry());
		local YCLtext = self:processEvent("COMBAT_LOG_EVENT", CombatLogGetCurrentEntry())
		-- NOTE: be sure to pass in nil for the color id or the color id may override the r, g, b values for this message
		--COMBATLOG:AddMessage( text, r, g, b, nil, true );
		_G[self.db.profile.outputFrame]:AddMessage( YCLtext, 1, 1, 1, nil, true );
		
		-- count can be 
		--  positive to advance from oldest to newest
		--  negative to advance from newest to oldest
		valid = CombatLogAdvanceEntry(-1)
		total = total + 1;
	end
	
	-- Show filtering progress bar
	CombatLogQuickButtonFrameProgressBar:SetValue(CombatLogQuickButtonFrameProgressBar:GetValue() + total);
	
	if ( not valid or (CombatLogQuickButtonFrameProgressBar:GetValue() >= COMBATLOG_MESSAGE_LIMIT) ) then
		CombatLogUpdateFrame.refiltering = false
		CombatLogUpdateFrame:SetScript("OnUpdate", nil)
		CombatLogQuickButtonFrameProgressBar:Hide();
	end
end
function YurrCombatLog:CombatLog_AddEvent()
end

function YurrCombatLog:setFilter()
	CombatLogResetFilter()
	
	if next(self.db.profile.watches) == nil then
		for k,v in ipairs(self.db.profile.filters) do
			local events = ""
			local sUnit = 0
			local dUnit = 0
			if v.active then
				for k2,v2 in pairs(v) do
					if k2 ~= "name" and k2 ~= "active" and k2 ~= "sourceUnit" and k2 ~= "destUnit" and v2 then
						events = events .. ", " .. k2
					end
				end
				for k2,v2 in pairs(v.sourceUnit) do
					if v2 then
						sUnit = bit.bor(sUnit, _G[k2])
					end
				end
				for k2,v2 in pairs(v.destUnit) do
					if v2 then
						dUnit = bit.bor(dUnit, _G[k2])
					end
				end
				if events == "" then sUnit = nil end
				if sUnit == 0 then sUnit = nil end
				if dUnit == 0 then dUnit = nil end
				CombatLogAddFilter(events, sUnit, dUnit)
			end
		end
	else
		for k,v in pairs(self.db.profile.watches) do
			CombatLogAddFilter("ENVIRONMENTAL_DAMAGE, SWING_DAMAGE, RANGE_DAMAGE, SPELL_DAMAGE, SPELL_HEAL, SPELL_PERIODIC_HEAL, SPELL_PERIODIC_DAMAGE, DAMAGE_SHIELD, SPELL_INSTAKILL SPELL_AURA_APPLIED, SPELL_AURA_REMOVED, UNIT_DIED", nil, v)
		end
	end
	self:Blizzard_CombatLog_Refilter()
end

function YurrCombatLog:AffiliationColor(flags)
	local color
	for mask in pairs( self.db.profile.unitColors ) do
		if ( CombatLog_Object_IsA (flags, mask) )then
			color = self.db.profile.unitColors[mask]
			break
		end
	end
	return color
end

function YurrCombatLog:ColorSpellBySchool(school, aura, suffix)
	local colors = {
		["_DISPEL"] = self.db.profile.spellColors[school],
		["_AURA_REMOVED"] = self.db.profile.otherColors.skill,
		["_AURA_REMOVED_DOSE"] = self.db.profile.otherColors.skill,
		["_MISSED"] = self.db.profile.otherColors.miss,
		["BUFF"] = self.db.profile.otherColors.buff,
		["DEBUFF"] = self.db.profile.otherColors.debuff,
		["_HEAL"] = self.db.profile.otherColors.heal,
		["_PERIODIC_HEAL"] = self.db.profile.otherColors.heal
	}
	
	if school == nil then
		Debug(1, "nil SPELLSCHOOL: ON suffix: " .. tostring(suffix))
	elseif not self.db.profile.spellColors[school] then
		Debug(1, "MISSING SPELLSCHOOL: " .. school .. " ON suffix: " .. suffix)
	end
	return colors[suffix] or colors[aura] or self.db.profile.spellColors[school]
end

function YurrCombatLog:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing)

	local resultStr
	
	if ( resisted or blocked or absorbed or critical or glancing or crushing ) then
		
		if ( resisted ) then
			if ( resisted < 0 ) then	--Its really a vulnerability
				resultStr = format(self.db.profile.format.VULNERABILITY, -resisted);
			else
				resultStr = format(self.db.profile.format.RESIST, resisted);
			end
		end
		if ( blocked ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..format(self.db.profile.format.BLOCK_VALUE, blocked);
			else
				resultStr = format(self.db.profile.format.BLOCK_VALUE, blocked);
			end
		end
		if ( absorbed ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..format(self.db.profile.format.ABSORB, absorbed);
			else
				resultStr = format(self.db.profile.format.ABSORB, absorbed);
			end
		end
		if ( glancing ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..self.db.profile.format.GLANCING;
			else
				resultStr = self.db.profile.format.GLANCING;
			end
		end
		if ( crushing ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..self.db.profile.format.CRUSHING;
			else
				resultStr = self.db.profile.format.CRUSHING;
			end
		end
		if ( critical ) then
			local critString = self.db.profile.format.CRITICAL;
			if ( spellId ) then
				critString = self.db.profile.format.CRITICAL_SPELL;
			end
			if ( resultStr ) then
				resultStr = resultStr.." "..critString;
			else
				resultStr = critString;
			end
		end
	end
	
	return resultStr

end

function YurrCombatLog:COMBAT_LOG_EVENT(...)
	local t = self:processEvent(...)
	_G[self.db.profile.outputFrame]:AddMessage(t)
end

function YurrCombatLog:processEvent(...)
	--Debug(1, ...)
	
	local timestamp = select(2, ...)
	local timestampStr
	
	local event = select(3, ...)
	
	local actionString
	
	local srcUnitName, srcFlags = select(5, ...)
	local srcUnitNameString
	
	local prefixString
	
	local spellName
	local spellNameString
	
	local suffixString
	
	local destUnitName, destFlags = select(8, ...)
	local destUnitNameString
	
	local amount
	local auraStack
	local overkill
	local overhealing
	local school
	local resisted
	local blocked
	local absorbed
	local critical
	local glancing
	local crushing
	local missType
	local amountMissed
	local powerType
	local extraSpellName
	local extraSchool
	local auraType
	local failedType
	local itemID
	local itemName
		
	local amountString
	local powerString
	local schoolStr
	local resultString
	
	local finalString
	
	if ( srcUnitName and CombatLog_Object_IsA(srcFlags, COMBATLOG_FILTER_MINE) ) then
			srcUnitName = UNIT_YOU_SOURCE;
	end
	if ( destUnitName and CombatLog_Object_IsA(destFlags, COMBATLOG_FILTER_MINE) ) then
			destUnitName = UNIT_YOU_DEST;
	end
	
	local base = 9
	local offset
	
	-- Prepare strings
	local prefix = strsub(event, 1, 5)
	local suffix = strsub(event, 6)
	
	-- Special events
	if event == "UNIT_DIED" then
		actionString = TUTORIAL_TITLE25 .. ": "
	elseif event == "UNIT_DESTROYED" then
		actionString = ACTION_UNIT_DESTROYED .. ": "
	elseif event == "ENCHANT_APPLIED" or event == "ENCHANT_REMOVED" then
		actionString = ENSCRIBE .. ": "
		offset = 0
		spellName, itemID, itemName = select(base + offset + 1, ...)
		school = 1
	elseif event == "PARTY_KILL" then
		spellName = ACTION_PARTY_KILL
		school = 1
	elseif event == "DAMAGE_SHIELD" or event == "DAMAGE_SPLIT" then
		offset = 3
		prefix = "SPELL"
		suffix = "_DAMAGE"
	elseif event == "DAMAGE_SHIELD_MISSED" then
		offset = 3
		prefix = "SPELL"
		suffix = "_MISSED"
	end
	
	if prefix == "SWING" then
		offset = 0
		spellName = MELEE
	elseif prefix == "SPELL" or prefix == "RANGE" then
		offset = 3
		spellName, school = select(11, ...)
	elseif prefix == "ENVIR" then
		offset = 1
		srcUnitName = destUnitName
		srcFlags = destFlags
		destUnitName = nil
		destFlags = nil
		local environmentalType = select(base + offset, ...)
		spellName = L[environmentalType]
		suffix = "_DAMAGE"
	end
	if suffix == "_DAMAGE" or suffix == "_PERIODIC_DAMAGE" or suffix == "_SPELL_BUILDING" then
		amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = select(base + offset + 1, ...)
	elseif suffix == "_MISSED" or suffix == "_PERIODIC_MISSED" then
		missType, amountMissed = select(base + offset + 1, ...)
	elseif suffix == "_HEAL" or suffix == "_PERIODIC_HEAL" then
		amount, overhealing, critical = select(base + offset + 1, ...)
	elseif suffix == "_ENERGIZE" or suffix == "_PERIODIC_ENERGIZE" then
		amount, powerType = select(base + offset + 1, ...)
	elseif suffix == "_DRAIN" or suffix == "_LEECH" or suffix == "_PERIODIC_DRAIN" or suffix == "_PERIODIC_LEECH" then
		amount, powerType, extraAmount = select(base + offset + 1, ...)
	elseif suffix == "_INTERRUPT" or suffix == "_DISPEL" or suffix == "_DISPEL_FAILED" or suffix == "_STOLEN" or suffix == "_AURA_BROKEN_SPELL" then
		--extraSpellID = select(base + offset + 1, ...)
		extraSpellName, extraSchool, auraType = select(base + offset + 2, ...)
	elseif suffix == "_EXTRA_ATTACKS" then
		amount = select(base + offset + 1, ...)
		spellName =  format(L.extraAttack, amount, spellName)
		school = 1
	elseif suffix == "_AURA_APPLIED" or suffix == "_AURA_REMOVED" or suffix == "_AURA_APPLIED_DOSE" or suffix == "_AURA_REMOVED_DOSE" or suffix == "_AURA_REFRESH" or suffix == "_AURA_BROKEN" then
		auraType, auraStack = select(base + offset + 1, ...)
	elseif suffix == "_CAST_FAILED" then
		failedType = select(base + offset + 1, ...)
	end
	
	if self.db.profile.blacklist[spellName] then return end
	
	-- Process strings
	if self.db.profile.timestamp then
		local stamp
		local left
		local right
		if self.db.profile.colorTimestamp then
			stamp = "|cFF" .. self.db.profile.otherColors.timestampColor .. date(self.db.profile.timestampFormat, timestamp) .. "|r"
		else
			stamp = date(self.db.profile.timestampFormat, timestamp)
		end
		if self.db.profile.braceTimestamp then
			if self.db.profile.colorTimestampBrace then
				left = "|cFF" .. self.db.profile.braceColors.Timestamp .. self.db.profile.braces.preTimestamp .. "|r"
				right = "|cFF" .. self.db.profile.braceColors.Timestamp .. self.db.profile.braces.postTimestamp .. "|r"
			else
				left = self.db.profile.braces.preTimestamp
				right = self.db.profile.braces.postTimestamp
			end
			timestampStr = left .. stamp .. right
		else
			timestampStr = stamp
		end
	else
		timestampStr = ""
	end
	
	if srcUnitName then
		local str
		local coloredUnitStr
		local leftBrace
		local rightBrace
		if self.db.profile.colorSrc then
			local color = self:AffiliationColor(srcFlags)
			coloredUnitStr = "|cFF" .. color .. srcUnitName .. "|r"
		end
		if self.db.profile.colorSrcBraces then
			leftBrace = "|cFF" .. self.db.profile.braceColors.Source .. self.db.profile.braces.preSource .. "|r"
			rightBrace = "|cFF" .. self.db.profile.braceColors.Source .. self.db.profile.braces.postSource .. "|r"
		else
			leftBrace = self.db.profile.braces.preSource
			rightBrace = self.db.profile.braces.postSource
		end
		if self.db.profile.braceSrcUnit then
			str = leftBrace .. "%s" .. rightBrace .. " "
		end
		srcUnitNameString = format(str or "%s ", coloredUnitStr or srcUnitName)
	else
		srcUnitNameString = ""
	end

	if self.db.profile.prefix then
		local str
		local prefixes = {
			["_AURA_APPLIED"] = self.db.profile["++"],
			["_AURA_APPLIED_DOSE"] = self.db.profile["++"],
			["_AURA_REMOVED"] = self.db.profile["--"],
			["_AURA_REMOVED_DOSE"] = self.db.profile["--"],
			["_AURA_REFRESH"] = self.db.profile["~~"],
			["_CAST_START"] = L.begins,
			["_CAST_SUCCESS"] = L.casts,
			["_CREATE"] = L.create,
			["_INSTAKILL"] = L.instantkill
		}
		str = prefixes[suffix] or ""
		if self.db.profile.colorPrefix and (str ~= "") then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			prefixString = format("|cFF%s%s|r ", color, str)
		elseif (str ~= "") then
			prefixString = str .. " "
		else
			prefixString = str
		end
	else
		prefixString = ""
	end
	
	if spellName then
		local coloredSpell
		local bracedSpell
		local leftBrace
		local rightBrace
		local stackedSpell
		if auraStack then stackedSpell = spellName .. "(" .. auraStack .. ")" else stackedSpell = spellName end
		
		if self.db.profile.colorSpell then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			coloredSpell = "|cFF" .. color .. stackedSpell .. "|r"
		else
			coloredSpell = stackedSpell
		end
		if self.db.profile.braceSpell then
			if self.db.profile.colorSpellBraces then
				leftBrace = "|cFF" .. self.db.profile.braceColors.Spell .. self.db.profile.braces.preSpell .. "|r"
				rightBrace = "|cFF" .. self.db.profile.braceColors.Spell .. self.db.profile.braces.postSpell .. "|r"
			else
				leftBrace = self.db.profile.braces.preSpell
				rightBrace = self.db.profile.braces.postSpell
			end
			
			bracedSpell = (leftBrace) .. "%s" .. (rightBrace) .. " "
		else
			bracedSpell = "%s "
		end
		spellNameString = format(bracedSpell, coloredSpell)
	else
		spellNameString = ""
	end
	
	if self.db.profile.suffixString then
		local str
		local result
		local suffixes = {
			["_HEAL"] = L.Heal,
			["_PERIODIC_HEAL"] = L.Regen,
			["_PERIODIC_DAMAGE"] = L.Dot,
			["_PERIODIC_MISSED"] = L.Dot,
			["_DISPEL"] = L.Dispel,
			["_DISPEL_FAILED"] = L.Dispel,
			["_INTERRUPT"] = L.Interrupt,
			["_SUMMON"] = L.Summon,
			["_RESURRECT"] = L.Resurrect,
			["MISS"] = self.db.profile.format.MISS
		}
		str = suffixes[suffix] or suffixes[missType] or failedType
		
		if str then
			suffixString = str .. " "
		else
			suffixString = ""
		end
	else
		suffixString = ""
	end
	
	if destUnitName then
		local str
		local coloredUnitStr
		local leftBrace
		local rightBrace
		if self.db.profile.colorDest then
			local color = self:AffiliationColor(destFlags)
			coloredUnitStr = "|cFF" .. color .. destUnitName .. "|r"
		end
		if self.db.profile.colorDestBraces then
			leftBrace = "|cFF" .. self.db.profile.braceColors.Dest .. self.db.profile.braces.preDest .. "|r"
			rightBrace = "|cFF" .. self.db.profile.braceColors.Dest .. self.db.profile.braces.postDest .. "|r"
		else
			leftBrace = self.db.profile.braces.preDest
			rightBrace = self.db.profile.braces.postDest
		end
		if self.db.profile.braceDestUnit then
			str = leftBrace .. "%s" .. rightBrace .. " "
		end
		destUnitNameString = format(str or "%s ", coloredUnitStr or destUnitName)
	else
		destUnitNameString = ""
	end
	
	if amount then
		local str
		local amountResult
		local coloredAmountStr
		local leftBrace
		local rightBrace
		local preDot
		local postDot
		local preCrit
		local postCrit
		
		if self.db.profile.overhealing and type(overhealing) == "number" and overhealing > 0 then
			amountResult = amount - overhealing .. "(+" .. overhealing .. ")"
		elseif self.db.profile.overkill and type(overkill) == "number" and overkill > 0 then
			amountResult = amount - overkill .. "(+" .. overkill .. ")"
		else
			amountResult = amount
		end
		
		if suffix == "_ENERGIZE" or suffix == "_PERIODIC_ENERGIZE" then
			amountResult = "+ " .. amountResult
		end
		
		
		if self.db.profile.colorAmount and school then
			local color = self:ColorSpellBySchool(school, auraType, suffix)
			coloredAmountStr = "|cFF" .. color .. amountResult .. "|r"
		end
		if self.db.profile.braceAmount then
			if self.db.profile.colorAmountBraces then
				leftBrace = "|cFF" .. self.db.profile.braceColors.Amount .. self.db.profile.braces.preAmount .. "|r"
				rightBrace = "|cFF" .. self.db.profile.braceColors.Amount .. self.db.profile.braces.postAmount .. "|r"
			else
				leftBrace = ""
				rightBrace = ""
			end
			str = "%s%s" .. leftBrace .. "%s" .. rightBrace .. "%s%s" .. " "
		end
		if self.db.profile.braceAmountDots and suffix == "_PERIODIC_DAMAGE" or suffix == "_PERIODIC_HEAL" then
			if colorAmountDotBraces then
				preDot = "|cFF" .. self.db.profile.braceColors.AmountDot .. self.db.profile.braces.preAmountDots .. self.db.profile.braceColors.AmountDot .. "|r"
				preDot = "|cFF" .. self.db.profile.braceColors.AmountDot .. self.db.profile.braces.postAmountDots .. self.db.profile.braceColors.AmountDot .. "|r"
			else
				preDot = self.db.profile.braces.preAmountDots
				postDot = self.db.profile.braces.postAmountDots
			end
		else
			preDot = ""
			postDot = ""
		end
		if self.db.profile.markCrits and critical then
			preCrit = self.db.profile.braces.preCrit
			postCrit = self.db.profile.braces.postCrit
		else
			preCrit = ""
			postCrit = ""
		end
		amountNameString = format(str or "%s%s%s%s%s ", preCrit, preDot, coloredAmountStr or amountResult, postDot, postCrit)
	else
		amountNameString = ""
	end
	if extraSpellName then
		local color = self:ColorSpellBySchool(extraSchool, auraType)
		amountNameString = "|cFF" .. color .. extraSpellName .. "|r "
	end
	if itemName then
		amountNameString = itemName
	end
	
	if self.db.profile.powerType and powerType then
		powerString = powerTypes[powerType] .. " "
	else
		powerString = ""
	end
	
	if self.db.profile.schoolText and school then
		local coloredSchool
		local bracedSchool
		if self.db.profile.colorSchoolText then
			local color = self:ColorSpellBySchool(school)
			coloredSchool = "|cFF" .. color .. schools[school] .. "|r"
		else
			coloredSchool = schools[school]
		end
		if self.db.profile.braceSchoolText then
			if self.db.profile.colorSchoolTextBraces then
				local color = self:ColorSpellBySchool(school)
				bracedSchool = "|cFF" .. color .. self.db.profile.braces.preSchoolText .. "|r" .. coloredSchool .. "|cFF" .. color .. self.db.profile.postSchoolText .. "|r"
			else
				bracedSchool = self.db.profile.braces.preSchoolText .. coloredSchool .. self.db.profile.postSchoolText
			end
		else
			bracedSchool = coloredSchool
		end
		schoolStr = bracedSchool
	else
		schoolStr = ""
	end
	
	if self.db.profile.resultString then
		if missType == "RESIST" then
			resisted = amountMissed
		elseif missType == "BLOCK" then
			blocked = amountMissed
		elseif missType == "ABSORB" then
			absorbed = amountMissed
		elseif missType ~= "MISS" then
			resultString = self.db.profile.format[missType]
		end
			
		if suffix == "_DISPEL_FAILED" then
			resultString = L.failed
		end
		
		if not resultString then resultString = self:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing) or "" end
	end
	
	finalString = timestampStr .. (actionString or "") .. srcUnitNameString .. prefixString .. spellNameString .. suffixString .. destUnitNameString .. amountNameString .. powerString .. schoolStr .. resultString
	return finalString
end

